@@ -18,9 +18,11 @@ namespace {
 const std::pair<llvm::StringRef, llvm::StringRef> IncludeMappings[] = {
     {"include/__stdarg___gnuc_va_list.h", "<cstdarg>"},
     {"include/__stdarg___va_copy.h", "<cstdarg>"},
+    {"include/__stdarg_header_macro.h", "<cstdarg>"},
     {"include/__stdarg_va_arg.h", "<cstdarg>"},
     {"include/__stdarg_va_copy.h", "<cstdarg>"},
     {"include/__stdarg_va_list.h", "<cstdarg>"},
+    {"include/__stddef_header_macro.h", "<cstddef>"},
     {"include/__stddef_max_align_t.h", "<cstddef>"},
     {"include/__stddef_null.h", "<cstddef>"},
     {"include/__stddef_nullptr_t.h", "<cstddef>"},
@@ -619,6 +621,9 @@ const std::pair<llvm::StringRef, llvm::StringRef> IncludeMappings[] = {
     {"sys/xattr.h", "<sys/xattr.h>"},
     {"bits/epoll.h", "<sys/epoll.h>"},
     {"bits/eventfd.h", "<sys/eventfd.h>"},
+    {"bits/getopt_core.h", "<getopt.h>"},
+    {"bits/getopt_ext.h", "<getopt.h>"},
+    {"bits/getopt_posix.h", "<getopt.h>"},
     {"bits/inotify.h", "<sys/inotify.h>"},
     {"bits/ipc.h", "<sys/ipc.h>"},
     {"bits/ipctypes.h", "<sys/ipc.h>"},
@@ -669,6 +674,7 @@ const std::pair<llvm::StringRef, llvm::StringRef> IncludeMappings[] = {
     {"bits/termios.h", "<termios.h>"},
     {"bits/types.h", "<sys/types.h>"},
     {"bits/types/siginfo_t.h", "<sys/siginfo.h>"},
+    {"bits/types/struct_iovec.h", "<sys/uio.h>"},
     {"bits/types/struct_itimerspec.h", "<sys/time.h>"},
     {"bits/types/struct_rusage.h", "<sys/resource.h>"},
     {"bits/uio.h", "<sys/uio.h>"},
